package de.tuberlin.emt.generator.actions;

import java.util.Iterator;
import java.util.List;


import org.eclipse.core.resources.IFile;
import org.eclipse.core.runtime.Platform;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EPackage;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.dialogs.IDialogSettings;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.window.Window;
import org.eclipse.jface.wizard.WizardDialog;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IObjectActionDelegate;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.IWorkbenchWizard;

import de.tuberlin.emt.generator.GeneratorPlugin;
import de.tuberlin.emt.generator.model.TransformationModelManager;
import de.tuberlin.emt.generator.wizard.GenerateRuleProjectWizard;
import de.tuberlin.emt.model.EMTFactory;
import de.tuberlin.emt.model.Transformation;
import de.tuberlin.emt.model.impl.EMTFactoryImpl;
import de.tuberlin.emt.model.util.EMTModelManager;


/**
 * Class : Generator <br/>
 * Package : de.tuberlin.emftransformation.generator <br/>
 * Project : de.tuberlin.emftransformation.generator <br/>
 * Created On : 20.12.2005 <br/>
 * Description : An action invoked when using the popup menu on a
 * transformation model file. <br/>
 * 
 * @author Enrico Biermann <br>
 * @author Guenter Kuhns <br>
 */
public class GenerateRuleAction implements IObjectActionDelegate {
	
	private Transformation transformation;
	private GenerateRuleProjectWizard wizard;
	private IWorkbench workbench;
	
	/**
	 * Constructor for Action1.
	 */
	public GenerateRuleAction() {
		super();
	}

	/**
	 * @see IObjectActionDelegate#setActivePart(IAction, IWorkbenchPart)
	 */
	public void setActivePart(IAction action, IWorkbenchPart targetPart) {
	}

	/**
	 * @see IActionDelegate#run(IAction)
	 */
	public void run(IAction action) {
		initWizard();
		runWizard();
	}

	private void initWizard(){
		wizard=new GenerateRuleProjectWizard(transformation);
        wizard.setForcePreviousAndNextButtons(false);
        wizard.setWindowTitle("Generator");
        ((IWorkbenchWizard) wizard).init(workbench,null);
    
        IDialogSettings workbenchSettings = GeneratorPlugin.getDefault().getDialogSettings();
        IDialogSettings wizardSettings = workbenchSettings
    								.getSection("NewWizardAction"); //$NON-NLS-1$
        if (wizardSettings == null)
        	wizardSettings = workbenchSettings.addNewSection("NewWizardAction"); //$NON-NLS-1$
        wizard.setDialogSettings(wizardSettings);	    
    }
	
	 private void runWizard(){
	    	IWorkbench workbench = GeneratorPlugin.getDefault().getWorkbench();
		 	Shell parent= workbench.getActiveWorkbenchWindow().getShell();
	        WizardDialog dialog = new WizardDialog(parent, wizard);
	        dialog.create();
	        workbench.getHelpSystem().setHelp(dialog.getShell(),
	        		GeneratorPlugin.ID +".new_wizard_context");
	        if(Window.OK== dialog.open()){}
	    }
	
	/**
	 * @see IActionDelegate#selectionChanged(IAction, ISelection)
	 */
	public void selectionChanged(IAction action, ISelection selection) {
		for (Iterator it = ((IStructuredSelection) selection).iterator(); it.hasNext();) {
			Object selected = it.next();
			if (selected instanceof IFile) {
				IFile file = (IFile) selected;
				if (file.exists())
				{
					URI fileUri = URI.createFileURI(Platform.getLocation().toString()+file.getFullPath().toString());
					EMTModelManager modelManager = new EMTModelManager(fileUri);
					List<String> packageUris = modelManager.getPackageURIs();
					for(Iterator<String> it2 = packageUris.iterator();it2.hasNext();){
						String packageUri = it2.next();
						if (!EPackage.Registry.INSTANCE.containsKey(packageUris.get(0))){
							System.out.println("Package "+packageUri+" not found in registry");
							EMTFactory fact = new EMTFactoryImpl();
							transformation = fact.createTransformation();
							return;
						}
					}
					try
					{
						modelManager.load();
						System.out.println("Transformation loaded successfully");
					}
					catch (Exception e)
					{
						System.out.println("Could not load transformation: "+e.getMessage());
						EMTFactory fact = new EMTFactoryImpl();
						transformation = fact.createTransformation();
					}
					transformation = modelManager.getTransformation();
				}
			}
		}
	}

	/**
	 * Method to start this action externally with a given transformation. 
	 * @param trans transformation that will be translated into rule classes
	 */
	public void start(Transformation trans){
		this.transformation = trans;
		initWizard();
		runWizard();
	}
}

/**
 * $Log: GenerateRuleAction.java,v $
 * Revision 1.1.2.1  2006/08/25 09:10:59  crema
 * merged with HEAD
 *
 * Revision 1.1  2006/07/17 12:13:55  enrico
 * Initial checkin
 *
 * Revision 1.4  2006/06/03 15:24:27  bunjip
 * Verwendung von Christians ModelManager zum Laden und Fehlerbehandlung
 *
 * Revision 1.3  2006/05/31 13:24:23  bunjip
 * Anpassungen an neues Modell
 *
 * Revision 1.2  2006/05/24 13:03:54  bunjip
 * letzte Veraenderungen uebernommen
 *
 * Revision 1.5  2006/05/16 11:57:05  enrico
 * Wizard angepasst
 *
 * Revision 1.4  2006/04/11 08:21:20  enrico
 * Progressmonitor hinzugefuegt
 *
 * Revision 1.3  2006/04/10 11:45:37  enrico
 * Wizard hinzugefuegt
 *
 * Revision 1.2  2006/03/20 01:26:59  bunjip
 * emfrule -> emftransformation
 *
 * Revision 1.1  2006/03/15 09:42:47  enrico
 * *** empty log message ***
 *
 * Revision 1.1  2006/03/14 17:06:10  eduardw
 * *** empty log message ***
 *
 * Revision 1.2  2006/01/05 14:08:24  enrico
 * TestProject->RuleProject
 *
 * Revision 1.1  2005/12/20 15:33:59  enrico
 * initial version
 *
 */
