package de.tuberlin.emt.generator.actions;

import java.io.BufferedReader;
import java.io.FileInputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.lang.reflect.InvocationTargetException;
import java.util.Iterator;
import java.util.Vector;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.IWorkspaceRoot;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.core.runtime.Platform;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.jobs.Job;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EPackage;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.ui.IActionDelegate;
import org.eclipse.ui.IObjectActionDelegate;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.actions.WorkspaceModifyOperation;
import org.osgi.framework.Constants;

import de.tuberlin.emt.generator.Generator;
import de.tuberlin.emt.model.EMTFactory;
import de.tuberlin.emt.model.Transformation;
import de.tuberlin.emt.model.impl.EMTFactoryImpl;
import de.tuberlin.emt.model.util.EMTModelManager;

/**
 * Class : GenerateRulePAckageAction <br/> Package :
 * de.tuberlin.emftransformation.generator <br/> Project :
 * de.tuberlin.emftransformation.generator <br/> Created On : 27.06.2006 <br/>
 * Description : An action invoked when using the popup menu on a transformation
 * model file. <br/>
 * 
 * @author Rene Schmutzler <br>
 */
public class GenerateRulePackageAction implements IObjectActionDelegate {

	private Transformation transformation;

	// private GenerateRuleProjectWizard wizard;
	private EMTModelManager modelManager;

	private Vector<IFile> files = new Vector<IFile>();

	private IProject project;

	/**
	 * Constructor for Action1.
	 */
	public GenerateRulePackageAction() {
		super();
	}

	/**
	 * @param ruleModel
	 * @param project
	 */
	public GenerateRulePackageAction(Transformation ruleModel, IProject project) {
		this.transformation = ruleModel;
		this.project = project;
	}

	/**
	 * @see IObjectActionDelegate#setActivePart(IAction, IWorkbenchPart)
	 */
	public void setActivePart(IAction action, IWorkbenchPart targetPart) {
	}

	/**
	 * @see IActionDelegate#run(IAction)
	 */
	public void run(IAction action) {
		generatePackage();
	}

	private void generatePackage() {
		if (project == null) {
			for (Iterator it = files.iterator(); it.hasNext();) {
				project = ((IFile) it.next()).getProject();
				GenerateRulePackageJob job = new GenerateRulePackageJob(project);
				ManifestUpdater updater = new ManifestUpdater();
				updater.packagePrefix=job.packagePrefix;
				try {
					updater.execute(new NullProgressMonitor());
				} catch (CoreException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				} catch (InvocationTargetException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				} catch (InterruptedException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
				job.schedule();
			}
		} else {
			GenerateRulePackageJob job = new GenerateRulePackageJob(project);
			ManifestUpdater updater = new ManifestUpdater();
			updater.packagePrefix=job.packagePrefix;
			try {
				updater.execute(new NullProgressMonitor());
			} catch (CoreException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (InvocationTargetException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (InterruptedException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			job.schedule();
		}
		
	}

	/**
	 * The worker method. It will find the container, create the file if missing
	 * or just replace its contents, and open the editor on the newly created
	 * file.
	 */

	private void doFinish(IProject project, IProgressMonitor monitor)
			throws CoreException {
		// create a sample file
		monitor.beginTask("Creating project", 5); //$NON-NLS-1$

		// if (!project.exists()) {
		// project.create(monitor);
		// project.open(null);
		// }

		// Delete .JetEmitters project in runtime-workspace
		try {
			IWorkspaceRoot root = ResourcesPlugin.getWorkspace().getRoot();
			IResource r = root.findMember(".JETEmitters"); //$NON-NLS-1$
			if (r != null)
				r.delete(false, monitor);
		} catch (CoreException e) {
		} catch (Exception e) {
		}

		Generator generator = new Generator(transformation, project.getName());
		// generator.start(monitor,project.getName());
		generator.start(monitor);

	}

	// private void initWizard(){
	// wizard=new GenerateRuleProjectWizard(transformation);
	// wizard.setForcePreviousAndNextButtons(false);
	// wizard.setWindowTitle("Generator");
	// ((IWorkbenchWizard) wizard).init(workbench,null);
	//    
	// IDialogSettings workbenchSettings =
	// GeneratorPlugin.getDefault().getDialogSettings();
	// IDialogSettings wizardSettings = workbenchSettings
	// .getSection("NewWizardAction"); //$NON-NLS-1$
	// if (wizardSettings == null)
	// wizardSettings = workbenchSettings.addNewSection("NewWizardAction");
	// //$NON-NLS-1$
	// wizard.setDialogSettings(wizardSettings);
	// }
	//	
	// private void runWizard(){
	// IWorkbench workbench = GeneratorPlugin.getDefault().getWorkbench();
	// Shell parent= workbench.getActiveWorkbenchWindow().getShell();
	// WizardDialog dialog = new WizardDialog(parent, wizard);
	// dialog.create();
	// workbench.getHelpSystem().setHelp(dialog.getShell(),
	// GeneratorPlugin.ID +".new_wizard_context");
	// if(Window.OK== dialog.open()){}
	// }

	/**
	 * @see IActionDelegate#selectionChanged(IAction, ISelection)
	 */
	public void selectionChanged(IAction action, ISelection selection) {
		for (Iterator it = ((IStructuredSelection) selection).iterator(); it
				.hasNext();) {
			Object selected = it.next();
			if (selected instanceof IFile) {
				IFile file = (IFile) selected;
				if (file.exists()) {
					URI fileUri = URI.createFileURI(Platform.getLocation()
							.toString()
							+ file.getFullPath().toString());
					modelManager = new EMTModelManager(fileUri);
					if (!modelManager.loadNeededPackages()) {
						EMTModelManager.showPackageNotFoundError();
					}
					// List<String> packageUris = modelManager.getPackageURIs();
					// for(Iterator<String> it2 =
					// packageUris.iterator();it2.hasNext();){
					// String packageUri = it2.next();
					// if
					// (!EPackage.Registry.INSTANCE.containsKey(packageUris.get(0))){
					// System.out.println("Package "+packageUri+" not found in
					// registry");
					// EMTFactory fact = new EMTFactoryImpl();
					// transformation = fact.createTransformation();
					// return;
					// }
					// }
					try {
						modelManager.load();
						files.add(file);
						// System.out.println("Transformation loaded
						// successfully"); //$NON-NLS-1$
					} catch (Exception e) {
						// System.out.println("Could not load transformation:
						// "+e.getMessage());
						EMTFactory fact = new EMTFactoryImpl();
						transformation = fact.createTransformation();
					}
					transformation = modelManager.getTransformation();
				}
			}
		}
	}

	private class ManifestUpdater extends WorkspaceModifyOperation {
		private String packagePrefix;

		private final int REQUIRE = 1;

		private final int EXPORT = 0;

		/*
		 * (non-Javadoc)
		 * 
		 * @see org.eclipse.ui.actions.WorkspaceModifyOperation#execute(org.eclipse.core.runtime.IProgressMonitor)
		 */
		@Override
		protected void execute(IProgressMonitor monitor) throws CoreException,
				InvocationTargetException, InterruptedException {
			try {
				if (packagePrefix == "") { //$NON-NLS-1$
					monitor.setTaskName("updating MANIFEST.MF"); //$NON-NLS-1$
					updateManifest("rules", monitor, EXPORT); //$NON-NLS-1$
					updateManifest("wrapper", monitor, EXPORT); //$NON-NLS-1$
				} else {
					updateManifest(packagePrefix + ".rules", monitor, EXPORT); //$NON-NLS-1$
					updateManifest(packagePrefix + ".wrapper", monitor, EXPORT); //$NON-NLS-1$
				}
				updateManifest("de.tuberlin.emt.common", monitor, REQUIRE); //$NON-NLS-1$
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}

		}

		private void updateManifest(String packageName,
				IProgressMonitor monitor, int type) throws CoreException,
				IOException {
			IFile manifest = project.getFile("META-INF\\MANIFEST.MF"); //$NON-NLS-1$
			if (manifest.exists()) {
				InputStream is = (FileInputStream) manifest.getContents();
				BufferedReader br = new BufferedReader(
						new InputStreamReader(is));
				String line;
				StringBuffer output = new StringBuffer();
				boolean containsPackage = false;
				try {
					while ((line = br.readLine()) != null) {
						if (line
								.trim()
								.replaceFirst(",", "").equalsIgnoreCase(packageName)) { //$NON-NLS-1$//$NON-NLS-2$
							containsPackage = true;
							break;
						}
						output.append(line + "\r\n"); //$NON-NLS-1$
						if (type == EXPORT) {
							if (line.startsWith(Constants.EXPORT_PACKAGE)) {
								if (line.endsWith(",")) { //$NON-NLS-1$
									output.append(" " + packageName + ",\r\n"); //$NON-NLS-1$ //$NON-NLS-2$
								} else {
									output.insert(
											output.lastIndexOf("\r\n"), ","); //$NON-NLS-1$ //$NON-NLS-2$
									output.append(" " + packageName + "\r\n"); //$NON-NLS-1$ //$NON-NLS-2$
								}
							}
						}
						if (type == REQUIRE) {
							if (line.startsWith(Constants.REQUIRE_BUNDLE)) {
								if (line.endsWith(",")) { //$NON-NLS-1$
									output.append(" " + packageName + ",\r\n"); //$NON-NLS-1$//$NON-NLS-2$
								} else {
									output.insert(
											output.lastIndexOf("\r\n"), ","); //$NON-NLS-1$ //$NON-NLS-2$
									output.append(" " + packageName + "\r\n"); //$NON-NLS-1$ //$NON-NLS-2$
								}
							}
						}
					}
				} catch (IOException e) {
					throw e;
				} finally {
					is.close();
				}

				if (!containsPackage) {
					IPath path = manifest.getLocation();
					FileWriter fw = new FileWriter(path.toFile());
					try {
						fw.write(output.toString());
						fw.flush();
					} catch (IOException e) {
						throw e;
					} finally {
						fw.close();
					}
					manifest.refreshLocal(IResource.DEPTH_INFINITE, monitor);
				}
			}
		}
	}

	private class GenerateRulePackageJob extends Job {
		private IProject project;

		private String packagePrefix;

		// private final int REQUIRE=1;
		// private final int EXPORT=0;
		/*
		 * the Constructor
		 */
		GenerateRulePackageJob(IProject project) {
			super("generating Rules"); //$NON-NLS-1$
			this.project = project;
			// TODO
			// this.packageSuffix=project.getName();
			EList packages = transformation.getPackages();
			for (Iterator iter = packages.iterator(); iter.hasNext();) {
				Object element = (Object) iter.next();
				if (element instanceof EPackage) {
					EPackage thePackage = (EPackage) element;
					this.packagePrefix = thePackage.getName();
					break;
				}
			}
			// this.packageSuffix=project.getName().toLowerCase();
		}

		@Override
		protected IStatus run(IProgressMonitor monitor) {
			try {
				doFinish(project, monitor);
				// if(packageSuffix==""){
				// monitor.setTaskName("updating MANIFEST.MF");
				// updateManifest("rules",monitor,EXPORT);
				// updateManifest("wrapper",monitor,EXPORT);
				// }
				// else{
				// updateManifest(packageSuffix+".rules",monitor,EXPORT);
				// updateManifest(packageSuffix+".wrapper",monitor,EXPORT);
				// }
				// updateManifest("de.tuberlin.emt.common",monitor,REQUIRE);
			} catch (CoreException e) {
				// System.err.println("Error during generating Rules");
				return Status.CANCEL_STATUS;
			}
			monitor.done();
			return Status.OK_STATUS;
		}
		// TODO complete comments
		// TODO check whether export or require exit
		/*
		 * this method updates the manifest of this project @param packageName
		 * @param monitor @param type should be <code>REQUIRE</code> or <code>EXPORT</code>
		 * to indicate the Section, where packageName will be added to @throws
		 * CoreException
		 * 
		 */
		// private void updateManifest(String packageName, IProgressMonitor
		// monitor,int type) throws CoreException, IOException{
		// IFile manifest=project.getFile("META-INF\\MANIFEST.MF");
		// if(manifest.exists()){
		// InputStream is=(FileInputStream)manifest.getContents();
		// BufferedReader br=new BufferedReader(new InputStreamReader(is));
		// String line;
		// StringBuffer output=new StringBuffer();
		// boolean containsPackage=false;
		// try {
		// while((line=br.readLine())!=null){
		// if(line.trim().replaceFirst(",", "").equalsIgnoreCase(packageName)){
		// containsPackage=true;
		// break;
		// }
		// output.append(line+"\r\n");
		// if(type==EXPORT){
		// if(line.startsWith("Export-Package")){
		// if(line.endsWith(",")){
		// output.append(" "+packageName+",\r\n");
		// }
		// else{
		// output.insert(output.lastIndexOf("\r\n"),",");
		// output.append(" "+packageName+"\r\n");
		// }
		// }
		// }
		// if(type==REQUIRE){
		// if(line.startsWith("Require-Bundle")){
		// if(line.endsWith(",")){
		// output.append(" "+packageName+",\r\n");
		// }
		// else{
		// output.insert(output.lastIndexOf("\r\n"),",");
		// output.append(" "+packageName+"\r\n");
		// }
		// }
		// }
		// }
		// }
		// catch(IOException e) {
		// throw e;
		// }
		// finally {
		// is.close();
		// }
		//
		// if(!containsPackage){
		// IPath path=manifest.getLocation();
		// FileWriter fw=new FileWriter(path.toFile());
		// try {
		// fw.write(output.toString());
		// fw.flush();
		// }
		// catch(IOException e) {
		// throw e;
		// }
		// finally {
		// fw.close();
		// }
		// manifest.refreshLocal(IResource.DEPTH_INFINITE, monitor);
		// }
		// }
		// }
	}
}
/**
 * $Log: GenerateRulePackageAction.java,v $
 * Revision 1.1.2.2.2.9  2007/01/27 10:58:57  reneschm
 * *** empty log message ***
 *
 * Revision 1.1.2.2.2.8  2007/01/19 22:19:18  reneschm
 * added missing assignment of package prefix to manifestUpdater
 *
 * Revision 1.1.2.2.2.7  2006/12/15 16:11:00  reneschm
 * reorganized manifestupdate
 * reformated
 * removed debuging syso
 * Revision 1.1.2.2.2.6 2006/12/15
 * 14:20:45 reneschm fixed manifest updating
 * 
 * Revision 1.1.2.2.2.5 2006/12/09 13:30:12 reneschm rollback
 * 
 * Revision 1.1.2.2.2.3 2006/11/15 17:27:59 reneschm changed the way of finding
 * the correct rootpackage name
 * 
 * Revision 1.1.2.2.2.2 2006/11/13 13:11:10 reneschm conversion vrom projectName
 * to packagePrefix now uses lowercases only
 * 
 * Revision 1.1.2.2.2.1 2006/10/29 17:54:35 reneschm added IOObjects merge with
 * head
 * 
 * Revision 1.1.2.2 2006/09/11 08:12:23 reneschm ne constructor to run this
 * action with e given ruleModel and a given project, where the generated code
 * should be placed
 * 
 * Revision 1.1.2.1 2006/07/24 17:36:51 reneschm transfered from old repository
 * location to new one and merged with latest version of emt
 * 
 * Revision 1.3.2.2 2006/06/29 13:03:30 reneschm *** empty log message ***
 * 
 */
