package de.tuberlin.emt.generator.data;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Vector;

import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EStructuralFeature;

import de.tuberlin.emt.model.Mapping;
import de.tuberlin.emt.model.ObjectStructure;
import de.tuberlin.emt.model.Rule;



public class EObjectData {
	EObject eObject;
	
	HashMap<EStructuralFeature,Object> rhsChanges = new HashMap<EStructuralFeature,Object>();
	Vector<HashMap<EStructuralFeature,Object>> nacChanges = new Vector<HashMap<EStructuralFeature,Object>>();
	
	EObjectData(Rule rule, EObject eObject) {
		this.eObject = eObject;
		for (Iterator it = rule.getMappings().iterator(); it.hasNext();) {
			Mapping mapping = (Mapping) it.next();
			if (mapping.getOrigin() == eObject) {
				EObject image = mapping.getImage();
				ObjectStructure owner = getOwner(rule, image);
				
				Vector<EStructuralFeature> differences = getDifferences(eObject,image);
				for (int i = 0; i < differences.size(); i++) {
					EStructuralFeature feature = differences.get(i);
					rhsChanges.put(feature,image.eGet(feature));
				}
				if (owner == rule.getRHS()) {
					for (int i = 0; i < differences.size(); i++) {
						EStructuralFeature feature = differences.get(i);
						rhsChanges.put(feature,image.eGet(feature));
					}
				}
				else {
					int nacIndex = getNacIndex(rule, owner);
					for (int i = 0; i < differences.size(); i++) {
						EStructuralFeature feature = differences.get(i);
						nacChanges.get(i).put(feature,image.eGet(feature));
					}
				}
			}
		}
	}

	
	private ObjectStructure getOwner(Rule rule, EObject eObject) {
		ObjectStructure lhs = rule.getLHS();
		for (Iterator it = lhs.getAllObjects().iterator(); it.hasNext();) {
			if (eObject == it.next())
				return lhs;
		}
		
		ObjectStructure rhs = rule.getLHS();
		for (Iterator it = rhs.getAllObjects().iterator(); it.hasNext();) {
			if (eObject == it.next())
				return rhs;
		}
		for (Iterator it = rule.getNAC().iterator(); it.hasNext();) {
			ObjectStructure nac = rule.getLHS();
			for (Iterator it2 = nac.getAllObjects().iterator(); it.hasNext();) {
				if (eObject == it2.next())
					return nac;
			}
		}
		return null;
	}
	
	private int getNacIndex(Rule rule, ObjectStructure nac) {
		int i = 0;
		for (Iterator it = rule.getNAC().iterator(); it.hasNext();) {
			if (it.next() == nac)
				return i;
			i++;
		}
		return -1;
	}
	
	private Vector<EStructuralFeature> getDifferences(EObject origin, EObject image) {
		Vector<EStructuralFeature> differences = new Vector<EStructuralFeature>();
		if(origin.eClass() == image.eClass()) {
			EClass eClass = origin.eClass();
			for (Iterator it = eClass.getEStructuralFeatures().iterator(); it.hasNext();) {
				EStructuralFeature feature = (EStructuralFeature) it.next();
				if (!origin.eGet(feature).equals(image.eGet(feature)))
					differences.add(feature);
			}
		}
		return differences;
	}
}
