package de.tuberlin.emt.generator.data;

/**
 * Class : PluginData <br/>
 * Package : de.tuberlin.emfrule.generator.util <br/>
 * Project : de.tuberlin.emfrule.generator <br/>
 * Created On : 09.01.2006 <br/>
 * Description : Wrapper class to extract plugin information <br/>
 */

public class PluginData{
	
	
	/**
	 * The id of the editor-plugin.
	 */
	private String projectName = new String("RuleProject");
	
	/**
	 * The version (default is 1.0.0) of the editor-plugin.
	 */
	private String version = new String("1.0.0");
	
	private String emfProjectName;
	
	/**
	 * The name of the editor-plugin.
	 */
	private String fName = "rules";
	
	/**
	 * The name of the library (&lt;name&gt;.jar) which is important <br>
	 * for building a plugin based on the editor-plugin. 
	 */
	private String libraryName ="/src/rules";
	
	/**
	 * The name of the source-folder, in which all *.java files are generator <br>
	 * for the editor-plugin.
	 */
	private String sourceFolderName = new String("D:\\eclipse31\\eclipse\\runtimeEclipseApplication\\RuleProject\\src");
	
	/**
	 * The name of the output-folder, in which all *.class files are generator <br>
	 * for the editor-plugin.
	 */
	private String outputFolderName = new String("D:\\eclipse31\\eclipse\\runtimeEclipseApplication\\RuleProject\\bin");
	
	
	
	/**
	 * The file extension is important for Eclipse, so that the editor can be <br>
	 * started correctly.
	 */
	private String fileExtension;
	
	/**
	 * The name of the base plugin class.
	 */
	private String className = "";	

	private String specname;

	
	/**
	 * @see tiger.generator.wizard.IPluginData#getProjectName()
	 */
	public String getProjectName() {
		return projectName;
	}
	
	/**
	 * @see tiger.generator.wizard.IPluginData#getVersion()
	 */
	public String getVersion() {
		return version;
	}
	
	/**
	 * @see tiger.generator.wizard.IPluginData#getName()
	 */
	public String getName() {
		return fName;
	}
	
	
	/**
	 * @see tiger.generator.wizard.IPluginData#getLibraryName()
	 */
	public String getLibraryName() {
		return libraryName;
	}
	
	
	/**
	 * @see tiger.generator.wizard.IPluginData#getSourceFolderName()
	 */
	public String getSourceFolderName() {
		return sourceFolderName;
	}
	
	
	/**
	 * @see tiger.generator.wizard.IPluginData#getOutputFolderName()
	 */
	public String getOutputFolderName() {
		return outputFolderName;
	}
	
	/**
	 * Sets the id for the project. Often it is the project name and the keyword "Plugin".
	 * @param id The project name.
	 */
	public void setProjectName(String id) {
		projectName = id;
	}
	
	/**
	 * Sets the name of the project. 
	 * @param name The projectname.
	 */
	public void setName(String name) {
		fName = name;
	}

	/**
	 * Sets the version for the editor plugin. It is 1.0.0.
	 * @param version the initial version
	 */
	public void setVersion(String version) {
		this.version = version;
	}
		
	/**
	 * Sets the name for the *.jar file if the editor plugin is exported.
	 * @param name
	 */
	public void setLibraryName(String name) {
		libraryName = name;
	}
	
	/**
	 * Sets the name for the source folder, in which the editor source code is stored.
	 * @param name The name for the source folder.
	 */
	public void setSourceFolderName(String name) {
		sourceFolderName = name;
	}
	
	/**
	 * @param name The name of the output folder.
	 */
	public void setOutputFolderName(String name) {
		outputFolderName = name;
	}
	
	/**
	 * @see tiger.generator.wizard.IPluginData#getFileExtension()
	 */
	public String getFileExtension() {
		return fileExtension;
	}
	/**
	 * @param fileExtension The fileExtension to set.
	 */
	public void setFileExtension(String fileExtension) {
		this.fileExtension = fileExtension;
	}
	
	
	/**
	 * @see tiger.generator.wizard.IPluginData#getClassname()
	 */
	public String getClassname() {
		return className;
	}
	
	/**
	 * @param classname
	 */
	public void setClassname(String classname) {
		className = classname;
	}

	public void setSpecName(String name) {
		this.specname = name;
		
	}
	public String getSpecName(){
		return specname;
	}

	public String getEmfProjectName() {
		return emfProjectName;
	}

	public void setEmfProjectName(String emfProjectName) {
		this.emfProjectName = emfProjectName;
	}
}