package de.tuberlin.emt.generator.wizard;

import java.lang.reflect.InvocationTargetException;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.IWorkspaceRoot;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.wizard.Wizard;
import org.eclipse.ui.INewWizard;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchWizard;
import org.eclipse.ui.actions.WorkspaceModifyOperation;

import de.tuberlin.emt.generator.Generator;
import de.tuberlin.emt.model.Transformation;


/**
 * This is a sample new wizard. Its role is to create a new file 
 * resource in the provided container. If the container resource
 * (a folder or a project) is selected in the workspace 
 * when the wizard is opened, it will accept it as the target
 * container. The wizard creates one file with the extension
 * "vls". If a sample multi-page editor (also available
 * as a template) is registered for the same extension, it will
 * be able to open it.
 */

public class GenerateRuleProjectWizard extends Wizard implements INewWizard {
	/**
	 */
	private GenerateRuleProjectWizardPage page;
	private String emfProjectName;
	private String packageName;
	/**
	 */
	private ISelection selection;
	
	private Transformation transformation;

	/**
	 * Constructor for TigerNewWizard.
	 */
	public GenerateRuleProjectWizard(Transformation transformation) {
		super();
		this.transformation = transformation;
		
		setNeedsProgressMonitor(true);
	}
	
	
	/**
	 * Adding the page to the wizard.
	 */

	public void addPages() {
		page = new GenerateRuleProjectWizardPage(packageName);
		addPage(page);
	}

	/**
	 * This method is called when 'Finish' button is pressed in
	 * the wizard. We will create an operation and run it
	 * using wizard as execution context.
	 */
	public boolean performFinish() {
		final IProject  project = page.getProjectHandle();
		
		WorkspaceModifyOperation op = new WorkspaceModifyOperation() {
			public void execute(IProgressMonitor monitor) throws InvocationTargetException {
				try {
					
					doFinish(project, monitor);
				} catch (CoreException e) {
					throw new InvocationTargetException(e);
				} finally {
					monitor.done();
				}
			}
		};
		try {
			getContainer().run(true, false, op);
		} catch (InterruptedException e) {
			return false;
		} catch (InvocationTargetException e) {
			Throwable realException = e.getTargetException();
			MessageDialog.openError(getShell(), "Error", realException.getMessage());
			return false;
		}
		return true;
	}
	
	/**
	 * The worker method. It will find the container, create the
	 * file if missing or just replace its contents, and open
	 * the editor on the newly created file.
	 */

	private void doFinish(
		IProject project,
		IProgressMonitor monitor)
		throws CoreException {
		// create a sample file
		monitor.beginTask("Creating project", 5);
		
		
		if (!project.exists()) {
			project.create(monitor);
			project.open(null);
		}
		
		//Delete .JetEmitters project in runtime-workspace
		try {
            IWorkspaceRoot root = ResourcesPlugin.getWorkspace().getRoot();
            IResource r=root.findMember(".JETEmitters");
            if(r!=null)
                r.delete(false,monitor);
	    } catch (CoreException e) {
	    }catch(Exception e){}
		
		
	    Generator generator = new Generator(transformation, project.getName());
		generator.start(monitor);
		
	}

	/**
	 * We will accept the selection in the workbench to see if
	 * we can initialize from it.
	 * @see IWorkbenchWizard#init(IWorkbench, IStructuredSelection)
	 */
	public void init(IWorkbench workbench, IStructuredSelection selection) {
		this.selection = selection;
	}
}